package com.ashokit.ticketbooking.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

@Configuration
@EnableWebSecurity //This Annotation will load securityconfig.java during startup as default security filter
public class SecurityConfig {
	
	
	//Authentication Logic
	/*
	 * Need to create two users 1) ashokitadmin/ashokitadmin ==> ADMIN
	 *                          2) ashokitguest/ashokitguest ==> GUEST
	 *                          
	 *       
	 *       in 2.x version the following code is used
	 *       public void populateAuthentication(AuthenticationManagerBuilder auth)
	 *       {
	 *       	auth.inMemoryAuthentication().user("ashokitadmin").password("ashokitadmin").role("ADMIN");
	 *       auth.inMemoryAuthentication().user("ashokitguest").password("ashokitguest").role("USER");
	 *       
	 *       }
	 *       
	 *       
	 *       In 3.x ==> We will use userDetailsService Object ==> Real Authentication Object
	 *                  UserDetails ==> Spring Boot Authentication Object
	 *                  UserDetailsService ==> Service which will manage Spring Boot Authentication Object
	 *                  
	 */
	
	@Bean
	public UserDetailsService userDetailsService(PasswordEncoder encoder) {
		
		UserDetails adminUser=User.withUsername("ashokitadmin").password(encoder.encode("ashokitadmin")).roles("ADMIN").build();
		
		UserDetails guestUser=User.withUsername("ashokitguest").password(encoder.encode("ashokitguest")).roles("GUEST").build();
		
		return new InMemoryUserDetailsManager(adminUser,guestUser);
				
	}
	
	
	@Bean
	public PasswordEncoder passwordEncoder() {
		return new BCryptPasswordEncoder();
	}
	
	
	
	
	//Authorization Logic
	/*
	 * 
	 * Authorization Logic ==> 1) ADMIN Role Can Access All Tickets
	 * 						   2) GUEST Role Can Access Individual Ticket
	 * 
	 */

	
	public SecurityFilterChain securityFilterChain(HttpSecurity authorizationObj) throws Exception {
		
		//All Tickets  ==> http://localhost:8080/tickets/all
		//formLogin ==> Redirect to Login Page for the first time
		
		//All Tickets Rule
		authorizationObj.csrf(csrf-> csrf.disable()). //1 or 2 days to explain
				authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/all/**"))
						.hasAnyRole("ADMIN")).formLogin();
		
		//Individual Ticket Rules
		authorizationObj.csrf(csrf-> csrf.disable()). //1 or 2 days to explain
		authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/ticket/**"))
				.hasAnyRole("ADMIN","GUEST")).formLogin();
		
		
		return authorizationObj.build();
	
		
	}
}
